package de.fernunihagen.rtt.ui.testadapter.jquery.internal;

import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;
import static de.fernunihagen.rtt.ui.testadapter.jquery.internal.RTTJQueryUIPlugin.getRTTJQueryUIPlugin;

import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.jobs.Job;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;

import org.eclipse.ui.IWorkingSet;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.IWorkingSetManager;

import ca.ubc.jquery.api.JQueryAPI;
import ca.ubc.jquery.api.JQueryException;
import ca.ubc.jquery.api.JQueryResult;

import de.fernunihagen.rtt.TestProcedureStep;

import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunSession;
import de.fernunihagen.rtt.framework.TestDescription;

import de.fernunihagen.rtt.framework.ITestElement.ProgressState;
import de.fernunihagen.rtt.framework.ITestElement.Result;

import de.fernunihagen.rtt.framework.TestDescription.ParamTypeInfo;

import de.fernunihagen.rtt.framework.testadapter.RefactoringTestDescription;
import de.fernunihagen.rtt.framework.testadapter.RefactoringTestRunState.TestAdapter;

import de.fernunihagen.rtt.internal.jdt.ExcludedJavaElementFilter;
import de.fernunihagen.rtt.internal.jdt.JavaTestElement;

import de.fernunihagen.rtt.internal.util.ICommand;

import de.fernunihagen.rtt.ui.internal.jdt.testadapter.AbstractJavaRefactoringTestAdapter;

import de.fernunihagen.rtt.ui.internal.util.CrossThreadOperation;

/**
 * JQuery refactoring test adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JQueryRefactoringTestAdapter extends
		AbstractJavaRefactoringTestAdapter {

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTJQueryUIPlugin()
				.getLogger(JQueryRefactoringTestAdapter.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}

	public JQueryRefactoringTestAdapter() {
		super();
	}

	private void reportError(final String message, final Throwable exception)
			throws CoreException {
		getRTTJQueryUIPlugin().reportError(message, exception);
	}

	private void reportError(final IStatus[] statuses, final String message)
			throws CoreException {
		getRTTJQueryUIPlugin().reportError(statuses, message);
	}

	private IStatus createError(final String message, final Throwable e) {
		return getRTTJQueryUIPlugin().createError(message, e);
	}

	@Pre("null != infos")
	private boolean allAnnotated(final ParamTypeInfo[] infos) {
		boolean fAllAnnotated = true;
		for (ParamTypeInfo info : infos) {
			fAllAnnotated = info.annotation != null;
			if (!fAllAnnotated) {
				break;
			}
		}
		return fAllAnnotated;
	}

	@Pre("null != test")
	private boolean hasQuery(final Method test) {
		return !JQueryRefactoringTestRunState.getQuery(test).isEmpty();
	}

	@Override
	@Pre
	protected boolean accept(final Method test, final Class<?> testcase) {
		return super.accept(test, testcase)
				&& (hasNoParameter(test) || (hasQuery(test) && allAnnotated(TestDescription
						.getTestParams(test))));
	}

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	private <T> T as(final Object o, final Class<? extends T> clazz) {
		return getClassUtil().as(o, clazz);
	}

	@Post("null != $return")
	private IWorkingSetManager getWorkingSetManager() {
		return PlatformUI.getWorkbench().getWorkingSetManager();
	}

	@Pre
	@Post
	@Override
	protected JQueryRefactoringTestRunState getState(final ITestContext context) {
		return (JQueryRefactoringTestRunState) super.getState(context);
	}

	@Pre("null != manager && null != javaProject")
	private void addProject(final IWorkingSetManager manager,
			final IJavaProject javaProject) {
		final CrossThreadOperation<Void> command = CrossThreadOperation
				.create(new ICommand<Void, Exception>() {
					@Override
					public Void execute() {
						final IProject project = javaProject.getProject();
						final String name = project.getName() + "/default";
						final IWorkingSet set = manager.getWorkingSet(name);
						if (null == set) {
							JQueryAPI.getFactBase().addProject(
									project.getProject());
						}
						// else already exists
						return null;
					}

				});

		try {
			command.execute();
		} catch (CoreException e) {
			getLogger().trace("Unexpected", e);
		}
	}

	@Pre("null != context")
	private void createFactbases(final ITestContext context,
			final IProgressMonitor pm) {
		try {
			final IWorkingSetManager manager = getWorkingSetManager();
			boolean fOk = context.getTestResult().isSuccess();
			final JQueryRefactoringTestRunState state = getState(context);
			final List<? extends IAdaptable> targets = state.getTargets();
			final ITestElementContainer container = context
					.createChildContainer(targets.size(), "Factbase");
			final SubMonitor monitor = SubMonitor.convert(pm,
					"Creating factbase(s)", targets.size() + 1);

			if (getLogger().isDebugEnabled()) {
				final String message = String.format(
						"Creating factbase(s) (%s)", targets.size());
				getLogger().debug(message);
			}

			container.setProgressState(ProgressState.RUNNING);

			try {
				Job.getJobManager().beginRule(JQueryAPI.getRule(), null);

				for (IAdaptable target : targets) {
					if (monitor.isCanceled()) {
						throw new OperationCanceledException();
					}

					final IJavaProject project = as(target, IJavaProject.class);
					Assert.isNotNull(project);

					monitor.subTask(project.getElementName());
					final JavaTestElement element = new JavaTestElement(
							container, project);

					element.setProgressState(ProgressState.RUNNING);

					if (fOk) {
						addProject(manager, project);
						element.setTestResult(Result.OK);
					}

					element.setProgressState(ProgressState.COMPLETED);
					monitor.worked(1);

					if (getLogger().isDebugEnabled()) {
						final String message = String.format(
								"%s (%s;%s;%.3fs)", element,
								element.getTestResult(),
								element.getProgressState(true),
								element.getElapsedTimeInSeconds());
						getLogger().debug(message);
					}
				}
			} finally {
				Job.getJobManager().endRule(JQueryAPI.getRule());
			}

			container.setProgressState(ProgressState.COMPLETED);

			if (getLogger().isDebugEnabled()) {
				final String message = String.format(
						"Creation done (%s;%s;%.3fs)",
						container.getTestResult(),
						container.getProgressState(true),
						container.getElapsedTimeInSeconds());
				getLogger().debug(message);
			}

		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	@Pre("null != manager && null != javaProject")
	private void removeProject(final IWorkingSetManager manager,
			final IJavaProject javaProject) {
		final CrossThreadOperation<Void> command = CrossThreadOperation
				.create(new ICommand<Void, Exception>() {
					@Override
					public Void execute() {
						final IProject project = javaProject.getProject();
						final String name = project.getName() + "/default";
						final IWorkingSet set = manager.getWorkingSet(name);
						if (null != set) {
							manager.removeWorkingSet(set);
						}
						return null;
					}
				});
		try {
			command.execute();
		} catch (CoreException e) {
			getLogger().trace("Unexpected", e);
		}
	}

	@Override
	@Pre
	public void start(final ITestContext context, final IProgressMonitor pm)
			throws CoreException {
		try {
			final SubMonitor monitor = SubMonitor.convert(pm,
					"Starting JQuery refactoring test adapter", 2);

			super.start(context, monitor.newChild(1));
			createFactbases(context, monitor.newChild(1));

		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	@Pre("null != args && null != container")
	private boolean acceptInputs(final Object[] args,
			final ITestElementContainer container) {
		if (null == args || args.length == 0) {
			return false;
		}

		final IElementFilter<IAdaptable> filter = container.getSession()
				.getDirectTargetFilter();
		for (Object arg : args) {
			final IAdaptable adaptable = as(arg, IAdaptable.class);
			if (adaptable == null || !filter.accept(adaptable)) {
				return false;
			}
		}

		return true;
	}

	@Pre("null != testadapter && null != container")
	@Post("null != $return")
	private Object[] getParameters(final TestAdapter testadapter,
			final ITestElementContainer container, final IProgressMonitor pm)
			throws CoreException {

		try {
			final JQueryRefactoringTestRunState state = (JQueryRefactoringTestRunState) testadapter
					.getRunState();
			final Method test = testadapter.getMethod();
			final ParamTypeInfo[] infos = TestDescription.getTestParams(test);
			final String[] vars = JQueryRefactoringTestRunState.getChosenVars(
					test, infos);
			final SubMonitor monitor = SubMonitor.convert(pm,
					"Collecting test input", vars.length);
			final JQueryResult result = state.getResult(test, vars);

			if (null != result) {
				final List<IStatus> errors = new ArrayList<IStatus>();
				final Object[] args = new Object[vars.length];

				final ITestElementContainer inputs = container
						.createChildContainer(vars.length, "Inputs");

				addStructuredChildElement(inputs, "This",
						testadapter.getInstanceID());

				inputs.setTestResult(Result.NONE);
				inputs.setProgressState(ProgressState.RUNNING);

				for (int i = 0; i < vars.length; i++) {
					if (monitor.isCanceled()) {
						throw new OperationCanceledException();
					}

					final String var = vars[i];
					final ITestElementContainer input = inputs
							.createChildContainer(1, var);

					input.setTestResult(Result.NONE);
					input.setProgressState(ProgressState.RUNNING);

					Object value = null;

					try {
						final Object key = result.get(var);
						if (null != key) {
							value = JQueryAPI.getJavaModelElement(key);

							final IJavaElement modelValue = as(value,
									IJavaElement.class);
							if (null != value) {
								final JavaTestElement element = new JavaTestElement(
										input, modelValue);
								element.setProgressState(ProgressState.COMPLETED);
								element.setTestResult(Result.NONE);
							} else {
								input.createChild("null", Result.IGNORED);
							}

							value = as(modelValue, infos[i].type);
						}
					} catch (JQueryException e) {
						input.setTestResult(Result.ERROR);
						errors.add(createError(
								"Failed to retrieve test input parameter", e));
					}

					args[i] = value;

					input.setProgressState(ProgressState.COMPLETED);

					if (getLogger().isDebugEnabled()) {
						final String message = String.format(
								"%s (%s;%s;%.3fs)", input,
								input.getTestResult(),
								input.getProgressState(true),
								input.getElapsedTimeInSeconds());
						getLogger().debug(message);
					}
				}

				inputs.setProgressState(ProgressState.COMPLETED);

				if (getLogger().isDebugEnabled()) {
					final String message = String.format("%s (%s;%s;%.3fs)",
							inputs, inputs.getTestResult(),
							inputs.getProgressState(true),
							inputs.getElapsedTimeInSeconds());
					getLogger().debug(message);
				}

				if (!errors.isEmpty()) {
					reportError(errors.toArray(new IStatus[errors.size()]),
							"At least one test input parameter could not been retrieved");
				}

				return args;

			} else {
				container.setTestResult(Result.IGNORED);
			}

			return new Object[0];

		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	@Pre("null != testadapter && null != container")
	private boolean runWithParameters(final TestAdapter testadapter,
			final ITestElementContainer container, final IProgressMonitor pm)
			throws CoreException {

		boolean fState;
		TestProcedureStep step = TestProcedureStep.skip();
		final JQueryRefactoringTestRunState state = (JQueryRefactoringTestRunState) testadapter
				.getRunState();

		try {
			final SubMonitor monitor = SubMonitor.convert(pm);
			final Object[] args = getParameters(testadapter, container,
					monitor.newChild(1));
			final ITestElementContainer inputs = !container.hasChildren() ? null
					: ((ITestElementContainer) container.getChildren()[0]);
			final String userDescription = state.getTestAdapter()
					.getDescription();
			final String description = String.format("%s%s", userDescription,
					formatInputs(inputs));

			container.setTarget(description);

			if (acceptInputs(args, container)) {
				step = (TestProcedureStep) testadapter.invoke(args);

				doTestProcedureStep(state, container, step, inputs,
						monitor.newChild(1));
			} else {
				// seek to next inputs in query selection
				step = TestProcedureStep.next();
			}

		} finally {
			if (null != pm) {
				pm.done();
			}

			fState = state.doTestProcedureStep(step);
		}

		return fState;
	}

	@Override
	@Pre
	protected boolean run(final TestAdapter testadapter,
			final ITestElementContainer element, final IProgressMonitor pm)
			throws CoreException {
		try {
			final Method test = testadapter.getMethod();
			final SubMonitor monitor = SubMonitor.convert(pm, "Executing test",
					1);

			monitor.subTask(test.getName());

			if (hasNoParameter(test)) {
				return runWithoutParameters(testadapter, element,
						monitor.newChild(1));
			}

			return runWithParameters(testadapter, element, monitor.newChild(1));

		} catch (OperationCanceledException e) {
			element.setProgressState(ProgressState.STOPPED);
			throw e;
		} catch (Exception e) {
			reportError("Test execution failed unexpected", e);
		} finally {
			if (null != pm) {
				pm.done();
			}
		}

		return true;
	}

	@Override
	@Pre
	@Post
	public JQueryRefactoringTestRunState create(final ITestRunSession session,
			final IProgressMonitor monitor) throws CoreException {

		final JQueryRefactoringTestRunState state;

		try {
			final IProject[] scope = session.getScope();
			final RefactoringTestDescription description = new RefactoringTestDescription(
					session.getConfiguration(this));
			final IElementFilter<IAdaptable> filter = new ExcludedJavaElementFilter(
					description.getExcludedElementIdentifiers());
			state = new JQueryRefactoringTestRunState(this, getCollector(
					filter, scope), description);
			state.updateTargets(monitor);

		} finally {
			if (null != monitor) {
				monitor.done();
			}
		}

		return state;
	}

}
