package de.fernunihagen.rtt.ui.internal.jdt;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getImageDescriptor;
import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.ISourceRange;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.compiler.IProblem;

import org.eclipse.jdt.ui.JavaUI;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import org.eclipse.ui.statushandlers.StatusManager;
import org.eclipse.ui.texteditor.ITextEditor;

import de.fernunihagen.rtt.framework.ITestElement;

import de.fernunihagen.rtt.framework.repository.DiffTestElement;

import de.fernunihagen.rtt.internal.jdt.JavaCompileProblemTestElement;
import de.fernunihagen.rtt.internal.jdt.JavaTestElement;
import de.fernunihagen.rtt.internal.jdt.JavaTestElementContainer;

import de.fernunihagen.rtt.internal.jdt.oracle.JUnitTestCaseElement;

import de.fernunihagen.rtt.ui.internal.action.AbstractActionAdapter;

/**
 * Open in editor action adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class OpenInEditorActionAdapter extends AbstractActionAdapter {

	private static IStatus createError(final String message, final Throwable e) {
		return getRTTCoreUIPlugin().createError(message, e);
	}

	private static void handleException(final CoreException e) {
		// Check for a nested CoreException
		IStatus status = e.getStatus();
		if (status != null && status.getException() instanceof CoreException) {
			status = ((CoreException) status.getException()).getStatus();
		}

		if (status == null) {
			StatusManager.getManager().handle(createError(e.getMessage(), e),
					StatusManager.SHOW);
		} else {
			StatusManager.getManager().handle(status, StatusManager.SHOW);
		}
	}

	@Post("null != $return")
	private IWorkspace getWorkspace() {
		return getRTTCoreUIPlugin().getWorkspace();
	}

	@Pre("null != problem")
	private IResource getProblemResource(final IProblem problem) {
		return getWorkspace().getRoot().findMember(
				String.valueOf(problem.getOriginatingFileName()));
	}

	@Override
	public IAction getTarget(final Object[] selected) {
		ITestElement testElement = null;
		for (Object element : selected) {
			if (canOpen(element)) {
				testElement = (ITestElement) element;
				break;
			}
		}
		if (null != testElement) {
			Object target = testElement.getTarget();
			final int offsetRef[] = { 0 };
			final int lengthRef[] = { 0 };

			if (target instanceof IResource) {
				target = JavaCore.create((IResource) target);
			}
			if (target instanceof IProblem) {
				final IProblem problem = (IProblem) target;
				target = JavaCore.create(getProblemResource(problem));
				offsetRef[0] = problem.getSourceStart();
				lengthRef[0] = problem.getSourceEnd()
						- problem.getSourceStart();
				if (offsetRef[0] < 0) {
					offsetRef[0] = 0;
				}
				if (lengthRef[0] != 0) {
					lengthRef[0]++;
				}
			}
			if (target instanceof IMember) {
				final IMember member = (IMember) target;
				final ISourceRange range;
				try {
					range = member.getNameRange();
					offsetRef[0] = range.getOffset();
					lengthRef[0] = range.getLength();
				} catch (JavaModelException e) {
					offsetRef[0] = lengthRef[0] = 0;
				}
			}

			final IJavaElement element = (IJavaElement) target;

			return new Action(getName(),
					getImageDescriptor("etool16/open_artifact_obj.gif")) {
				@Override
				public void run() {
					IEditorPart editor;
					try {
						editor = JavaUI.openInEditor(element);
						if (editor != null && editor instanceof ITextEditor) {
							((ITextEditor) editor).selectAndReveal(
									offsetRef[0], lengthRef[0]);
						}
					} catch (PartInitException e) {
						handleException(e);
					} catch (JavaModelException e) {
						handleException(e);
					}
				}
			};
		}
		return null;
	}

	private boolean isJavaResource(final IResource resource) {
		return null != resource
				&& JavaCore.isJavaLikeFileName(resource.getName());
	}

	private boolean canOpenAsJUnitTestElement(final Object selected) {
		return selected instanceof JUnitTestCaseElement
				&& null != ((JUnitTestCaseElement) selected).getTarget();
	}

	private boolean canOpenAsJavaTestElement(final Object selected) {
		return selected instanceof JavaTestElement
				|| selected instanceof JavaTestElementContainer;
	}

	private boolean canOpenAsJavaCompileProblemTestElement(final Object selected) {
		return selected instanceof JavaCompileProblemTestElement
				&& null != getProblemResource(((JavaCompileProblemTestElement) selected)
						.getTarget());
	}

	private boolean canOpenAsDiffTestElement(final Object selected) {
		return selected instanceof DiffTestElement
				&& isJavaResource((IResource) ((DiffTestElement) selected)
						.getTarget());
	}

	private boolean canOpen(final Object selected) {
		return null != selected
				&& (canOpenAsJavaTestElement(selected)
						|| canOpenAsJavaCompileProblemTestElement(selected)
						|| canOpenAsDiffTestElement(selected) || canOpenAsJUnitTestElement(selected));
	}

	@Override
	public boolean isApplicable(Object[] selected) {
		for (Object element : selected) {
			if (canOpen(element)) {
				return true;
			}
		}
		return false;
	}

}
