package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import org.contract4j5.contract.Contract;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IPackageFragment;

import org.eclipse.jface.viewers.ViewerCell;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Control;

import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;
import de.fernunihagen.rtt.ui.internal.util.ViewerField;

/**
 * Test subject field resource.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestSubjectFieldResource extends ViewerField {

	@Override
	public String getColumnHeaderText() {
		return "Resource";
	}

	@Override
	public String getID() {
		return "resource";
	}

	@Override
	public int getDefaultColumnWidth(final Control control) {
		return 4 * super.getDefaultColumnWidth(control);
	}

	private Image getImageByModelType(IJavaElement javaElement) {
		switch (javaElement.getElementType()) {
		case IJavaElement.JAVA_PROJECT:
			return RTTCoreUIPlugin.createImage("eview16/projects.gif");
		case IJavaElement.PACKAGE_FRAGMENT_ROOT:
			return RTTCoreUIPlugin.createImage("eview16/packages.gif");
		case IJavaElement.PACKAGE_FRAGMENT:
			return RTTCoreUIPlugin.createImage("eview16/package.gif");
		case IJavaElement.COMPILATION_UNIT:
			return RTTCoreUIPlugin.createImage("eview16/source.gif");
		case IJavaElement.TYPE:
			return RTTCoreUIPlugin.createImage("eview16/types.gif");
		case IJavaElement.FIELD:
		case IJavaElement.METHOD:
		case IJavaElement.INITIALIZER:
			return RTTCoreUIPlugin.createImage("eview16/members.gif");
		default:
		case IJavaElement.JAVA_MODEL:
		case IJavaElement.CLASS_FILE:
		case IJavaElement.PACKAGE_DECLARATION:
		case IJavaElement.IMPORT_CONTAINER:
		case IJavaElement.IMPORT_DECLARATION:
		case IJavaElement.LOCAL_VARIABLE:
		case IJavaElement.TYPE_PARAMETER:
		case IJavaElement.ANNOTATION:
			return null;
		}
	}

	@Override
	public void update(final ViewerCell cell) {
		final AbstractTestSubject element = (AbstractTestSubject) cell
				.getElement();
		final IJavaElement javaElement = element.getJavaElement();

		cell.setImage(getImageByModelType(javaElement));

		String text = javaElement.getElementName();
		if (text.isEmpty() && javaElement instanceof IPackageFragment) {
			text = "(default)";
		}
		cell.setText(text.replace('\n', SWT.LF));
	}

}
