package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ViewerCell;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import de.fernunihagen.rtt.framework.ITestActor;

import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;
import de.fernunihagen.rtt.ui.internal.jdt.launcher.AbstractTestSubject.ExcludedState;
import de.fernunihagen.rtt.ui.internal.util.ViewerField;

/**
 * Test subject field selection.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestSubjectFieldSelection extends ViewerField {
	@Invar("null != $target")
	private final ITestActor _actor;

	private static final Color INACTIVE = new Color(RTTCoreUIPlugin
			.getDefault().getWorkbench().getDisplay(), 180, 180, 180);

	private final class SelectionEditingSupport extends EditingSupport {
		@Invar("null != $target")
		private final CellEditor _editor;

		@Pre("null != viewer")
		public SelectionEditingSupport(final ColumnViewer viewer) {
			super(viewer);
			_editor = new CheckboxCellEditor(null, SWT.CHECK | SWT.READ_ONLY);
		}

		@Override
		protected boolean canEdit(final Object element) {
			final AbstractTestSubject testelement = (AbstractTestSubject) element;
			return testelement.isInSync()
					&& testelement.getExcludedState(_actor) != ExcludedState.IGNORED;
		}

		@Override
		protected CellEditor getCellEditor(final Object element) {
			return _editor;
		}

		@Override
		protected Object getValue(final Object element) {
			final AbstractTestSubject testelement = (AbstractTestSubject) element;
			return testelement.isExcluded(_actor);
		}

		private void updateChildren(final AbstractTestSubject testelement) {
			final AbstractTestSubject[] children = testelement.getChildren();
			for (AbstractTestSubject element : children) {
				updateChildren(element);
			}
			getViewer().update(testelement, null);
		}

		private void updateSelfAndParents(AbstractTestSubject testelement) {
			while (null != testelement) {
				getViewer().update(testelement, null);
				testelement = testelement.getParent();
			}
		}

		@Post("null != $return")
		private Display getDisplay() {
			Display display;
			display = Display.getCurrent();
			if (display == null) {
				display = Display.getDefault();
			}
			return display;
		}

		@Override
		protected void setValue(final Object element, final Object value) {
			final AbstractTestSubject testelement = (AbstractTestSubject) element;

			BusyIndicator.showWhile(getDisplay(), new Runnable() {
				@Override
				public void run() {
					testelement.exclude(_actor, (Boolean) value);
					testelement.notifyChanges();
				}
			});

			updateChildren(testelement);
			updateSelfAndParents(testelement);
		}
	}

	/**
	 * Creates a test subject field selection.
	 * 
	 * @param actor
	 *            Test actor
	 */
	@Pre("null != actor")
	public TestSubjectFieldSelection(final ITestActor actor) {
		_actor = actor;
	}

	@Override
	public EditingSupport getEditingSupport(final ColumnViewer viewer) {
		return new SelectionEditingSupport(viewer);
	}

	@Override
	public String getColumnHeaderText() {
		return _actor.getName();
	}

	@Override
	public String getID() {
		return _actor.getID();
	}

	@Override
	public void update(final ViewerCell cell) {
		final AbstractTestSubject element = (AbstractTestSubject) cell
				.getElement();
		String text = "";
		if (!element.isInSync()) {
			text = "Ignored (Out of Sync)";
			cell.setImage(RTTCoreUIPlugin.createImage("etool16/desel_obj.gif"));
			cell.setForeground(INACTIVE);
		} else {
			final ExcludedState state = element.getExcludedState(_actor);
			switch (state) {
			case ALL:
				cell.setImage(RTTCoreUIPlugin
						.createImage("etool16/desel_obj.gif"));
				text = "Excluded";
				break;
			case IGNORED:
				cell.setImage(RTTCoreUIPlugin
						.createImage("etool16/desel_obj.gif"));
				text = "Ignored";
				cell.setForeground(INACTIVE);
				break;
			case PARTIAL:
				cell.setImage(RTTCoreUIPlugin
						.createImage("etool16/selpart_obj.gif"));
				text = "Included";
				break;
			case NONE:
				cell.setImage(RTTCoreUIPlugin
						.createImage("etool16/sel_obj.gif"));
				text = "Included";
				break;
			}
		}
		cell.setText(text);
	}
}
