package de.fernunihagen.rtt.ui.internal.jdt.testadapter;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import java.lang.reflect.Method;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubMonitor;

import org.eclipse.jdt.core.IJavaProject;

import de.fernunihagen.rtt.TestProcedureStep;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.ITestElement.ProgressState;

import de.fernunihagen.rtt.framework.testadapter.RefactoringTestRunState;
import de.fernunihagen.rtt.framework.testadapter.RefactoringTestRunState.TestAdapter;

/**
 * Java refactoring test adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaRefactoringTestAdapter extends
		AbstractJavaRefactoringTestAdapter {

	private static final IAdaptable[] EMPTY_ADAPTABLES = new IAdaptable[0];
	private static final IProject[] EMPTY_PROJECTS = new IProject[0];
	private static final IJavaProject[] EMPTY_JAVAPROJECTS = new IJavaProject[0];

	/**
	 * Create a java refactoring test adapter.
	 */
	public JavaRefactoringTestAdapter() {
		super();
	}

	private void reportError(final String message, final Throwable exception)
			throws CoreException {
		getRTTCoreUIPlugin().reportError(message, exception);
	}

	private boolean hasOneParameter(final Method test) {
		return test.getParameterTypes().length == 1;
	}

	private boolean hasOneParameterAdaptableArray(final Method test) {
		return hasOneParameter(test)
				&& test.getParameterTypes()[0].isArray()
				&& test.getParameterTypes()[0]
						.isAssignableFrom(EMPTY_ADAPTABLES.getClass());
	}

	private boolean hasOneParameterProjectArray(final Method test) {
		return hasOneParameter(test)
				&& test.getParameterTypes()[0].isArray()
				&& test.getParameterTypes()[0].isAssignableFrom(EMPTY_PROJECTS
						.getClass());
	}

	private boolean hasOneParameterJavaProjectArray(final Method test) {
		if (!hasOneParameter(test)) {
			return false;
		}

		Class<?> clazz = test.getParameterTypes()[0];
		return clazz.isAssignableFrom(EMPTY_JAVAPROJECTS.getClass());
	}

	@Override
	@Pre
	protected boolean accept(final Method test, final Class<?> testcase) {
		return super.accept(test, testcase)
				&& (hasNoParameter(test)
						|| hasOneParameterJavaProjectArray(test)
						|| hasOneParameterProjectArray(test) || hasOneParameterAdaptableArray(test));
	}

	@Pre("null != testadapter && null != element && null != arg")
	private boolean runWithOneParameter(final Object arg,
			final TestAdapter testadapter, final ITestElementContainer element,
			final IProgressMonitor pm) throws CoreException {
		final RefactoringTestRunState state = testadapter.getRunState();
		final ITestElementContainer inputs = element.createChildContainer(1,
				"Inputs");
		addStructuredChildElement(inputs, "This", testadapter.getInstanceID());
		addStructuredChildElement(inputs, "Parameter", arg.toString());
		inputs.setProgressState(ProgressState.COMPLETED);

		final TestProcedureStep step = (TestProcedureStep) testadapter
				.invoke(arg);

		return doTestProcedureStep(state, element, step, inputs, pm);
	}

	@Pre("null != testadapter && null != element")
	private boolean runWithOneParameterJavaProjectArray(
			final TestAdapter testadapter, final ITestElementContainer element,
			final IProgressMonitor pm) throws CoreException {

		final RefactoringTestRunState state = testadapter.getRunState();
		return runWithOneParameter(state.getTargets().toArray(
				new IJavaProject[0]), testadapter, element, pm);
	}

	@Pre("null != testadapter && null != element")
	private boolean runWithOneParameterAdaptableArray(
			final TestAdapter testadapter, final ITestElementContainer element,
			final IProgressMonitor pm) throws CoreException {

		final RefactoringTestRunState state = testadapter.getRunState();
		return runWithOneParameter(state.getTargets()
				.toArray(new IAdaptable[0]), testadapter, element, pm);
	}

	@Pre("null != testadapter && null != element")
	private boolean runWithOneParameterProjectArray(
			final TestAdapter testadapter, final ITestElementContainer element,
			final IProgressMonitor pm) throws CoreException {

		final RefactoringTestRunState state = testadapter.getRunState();
		return runWithOneParameter(state.getScope(), testadapter, element, pm);
	}

	@Override
	@Pre
	protected boolean run(final TestAdapter testadapter,
			final ITestElementContainer element, final IProgressMonitor pm)
			throws CoreException {

		try {
			final Method test = testadapter.getMethod();
			final SubMonitor monitor = SubMonitor.convert(pm, "Executing test",
					1);

			monitor.subTask(test.getName());

			if (hasOneParameterJavaProjectArray(test)) {
				return runWithOneParameterJavaProjectArray(testadapter,
						element, monitor.newChild(1));
			}
			if (hasOneParameterProjectArray(test)) {
				return runWithOneParameterProjectArray(testadapter, element,
						monitor.newChild(1));
			}
			if (hasOneParameterAdaptableArray(test)) {
				return runWithOneParameterAdaptableArray(testadapter, element,
						monitor.newChild(1));
			}

			return runWithoutParameters(testadapter, element, monitor
					.newChild(1));

		} catch (OperationCanceledException e) {
			element.setProgressState(ProgressState.STOPPED);
			throw e;
		} catch (Exception e) {
			reportError("Test execution failed unexpected", e);
		} finally {
			if (null != pm) {
				pm.done();
			}
		}

		return true;
	}
}
