package de.fernunihagen.rtt.ui.internal.view;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;
import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getDefault;

import static de.fernunihagen.rtt.internal.ResourceUtil.getResourceUtil;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import java.lang.reflect.InvocationTargetException;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import org.eclipse.help.IContext;
import org.eclipse.help.IContextProvider;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;

import org.eclipse.jface.dialogs.ProgressMonitorDialog;

import org.eclipse.jface.operation.IRunnableWithProgress;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.jface.util.PropertyChangeEvent;

import org.eclipse.jface.viewers.ColumnPixelData;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.ltk.core.refactoring.RefactoringDescriptorProxy;

import org.eclipse.swt.SWT;

import org.eclipse.swt.events.HelpEvent;
import org.eclipse.swt.events.HelpListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TreeAdapter;
import org.eclipse.swt.events.TreeEvent;

import org.eclipse.swt.graphics.Font;

import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;

import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.OpenAndLinkWithEditorHelper;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import org.eclipse.ui.part.ViewPart;

import org.eclipse.ui.statushandlers.StatusManager;

import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElement;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunSession;
import de.fernunihagen.rtt.framework.ITestElement.ProgressState;
import de.fernunihagen.rtt.framework.ITestElement.Result;
import de.fernunihagen.rtt.framework.ITestElement.RunStage;

import de.fernunihagen.rtt.framework.model.ExceptionTestElement;
import de.fernunihagen.rtt.framework.model.TestRootRunSession;

import de.fernunihagen.rtt.framework.oracle.IOracle;

import de.fernunihagen.rtt.framework.testadapter.IRefactoringTestAdapter;
import de.fernunihagen.rtt.framework.testadapter.RefactoringTestElement;

import de.fernunihagen.rtt.internal.jdt.JavaCompileProblemTestElement;

import de.fernunihagen.rtt.internal.jdt.oracle.JUnitTestCaseElement;

import de.fernunihagen.rtt.internal.runtime.AbstractTestApplication;

import de.fernunihagen.rtt.ui.IActionAdapter;
import de.fernunihagen.rtt.ui.RTTCoreUI;

import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;

import de.fernunihagen.rtt.ui.internal.util.ViewerField;
import de.fernunihagen.rtt.ui.internal.util.ViewerFieldLabelProvider;

/**
 * Test result view.
 * 
 * @author Osama El Hosami
 */
@Contract
public class TestResultView extends ViewPart {
	private boolean _fDisposed;
	private TreeViewer _viewer;
	private IAction _save;
	private IAction _rerunAll;
	private IAction _expandAll;
	private IAction _collapseAll;
	private IAction _autoFullExpand;
	private IAction _hideSuccess;
	private IAction _hideIgnored;
	private IAction _hideFailures;
	private IAction _hideErrors;
	private IAction _hideInitialization;
	private Label _label;

	private static final String FIELD = "FIELD";

	/**
	 * View part identifier
	 */
	public static final String ID = "de.fernunihagen.rtt.ui.view.testresult";

	private static final class SuccessFilter extends ViewerFilter {
		@Override
		public boolean select(final Viewer viewer, final Object parentElement,
				final Object element) {
			if (element != null && element instanceof ITestElement) {
				return ((ITestElement) element).getTestResult() != Result.OK;
			}
			return false;
		}
	}

	private static final class IgnoredFilter extends ViewerFilter {
		@Override
		public boolean select(final Viewer viewer, final Object parentElement,
				final Object element) {
			if (element != null && element instanceof ITestElement) {
				return ((ITestElement) element).getTestResult() != Result.IGNORED;
			}
			return false;
		}
	}

	private static final class FailureFilter extends ViewerFilter {
		@Override
		public boolean select(final Viewer viewer, final Object parentElement,
				final Object element) {
			if (element != null && element instanceof ITestElement) {
				return ((ITestElement) element).getTestResult() != Result.FAILURE;
			}
			return false;
		}
	}

	private static final class ErrorFilter extends ViewerFilter {
		@Override
		public boolean select(final Viewer viewer, final Object parentElement,
				final Object element) {
			if (element != null && element instanceof ITestElement) {
				return ((ITestElement) element).getTestResult() != Result.ERROR;
			}
			return false;
		}
	}

	private static final class InitializationFilter extends ViewerFilter {
		private boolean isFirstEntry(ITestElement element) {
			final ITestRunSession session = element.getSession();
			return session.hasChildren() && session.getChildren()[0] == element;
		}

		@Override
		public boolean select(final Viewer viewer, final Object parentElement,
				final Object object) {
			if (object != null && object instanceof ITestElement) {
				ITestElement element = (ITestElement) object;
				if (isFirstEntry(element)) {
					return !element.getTestResult().isSuccess();
				}
				return element.getStage() == RunStage.RUN
						|| !element.getTestResult().isSuccess();
			}
			return false;
		}
	}

	public static final class TestResultViewFilterHolder {
		public static final ViewerFilter _SUCCESS_FILTER = new SuccessFilter();
		public static final ViewerFilter _FAILURES_FILTER = new FailureFilter();
		public static final ViewerFilter _IGNORED_FILTER = new IgnoredFilter();
		public static final ViewerFilter _ERRORS_FILTER = new ErrorFilter();
		public static final ViewerFilter _INITIALIZATION_FILTER = new InitializationFilter();
	}

	private static final class TestResultViewFieldHolder {
		public static final ViewerField _lABEL = new TestElementFieldLabel();
		public static final ViewerField _PROGRESSSTATE = new TestElementFieldProgressState();
		public static final ViewerField _TIME = new TestElementFieldTime();
		public static final ViewerField _RESULT = new TestElementFieldResult();
		public static final ViewerField _CLASS = new TestElementFieldClass();
		public static final ViewerField _RUNSTAGE = new TestElementFieldRunStage();
		public static final ViewerField[] _ALL = new ViewerField[] {
				TestResultViewFieldHolder._lABEL,
				TestResultViewFieldHolder._TIME,
				TestResultViewFieldHolder._PROGRESSSTATE,
				TestResultViewFieldHolder._RESULT,
				TestResultViewFieldHolder._CLASS,
				TestResultViewFieldHolder._RUNSTAGE };
	}

	private static IStatus createError(final String message, final Throwable e) {
		return getRTTCoreUIPlugin().createError(message, e);
	}

	@Post("null != $return")
	private ViewerField[] getResultVisibleFields() {
		return TestResultViewFieldHolder._ALL;
	}

	// private SelectionListener getHeaderListener() {
	// return new SelectionAdapter() {
	// @Override
	// public void widgetSelected(final SelectionEvent e) {
	// final TreeColumn column = (TreeColumn) e.widget;
	// final ViewerField field = (ViewerField) column.getData(FIELD);
	//
	// // TODO SORT
	// }
	// };
	// }

	private IAction createAction(final Action action, final String text,
			final String image) {
		action.setText(text);
		action.setToolTipText(text);
		action.setImageDescriptor(RTTCoreUIPlugin.getImageDescriptor(image));
		return action;
	}

	private IAction createAction(final String text, final int style,
			final IPropertyChangeListener listener) {
		Action action = new Action(text, style) {
		};

		action.setText(text);
		action.setToolTipText(text);
		if (null != listener) {
			action.addPropertyChangeListener(listener);
		}
		return action;
	}

	private void saveSummary() {
		final DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
		final FileDialog dialog = new FileDialog(getSite().getShell(), SWT.SAVE);
		dialog.setFilterNames(new String[] { "Comma-separated values",
				"All files (*.*)" });
		dialog.setFilterExtensions(new String[] { "*.cvs", "*.*" });
		dialog.setFilterPath(null);
		dialog.setOverwrite(true);
		dialog.setFileName(String.format("rttsummary%s.csv",
				dateFormat.format(new Date())));

		final String filename = dialog.open();
		if (null == filename) {
			return;
		}

		Writer writer = null;
		try {
			final File file = new File(filename);
			writer = new BufferedWriter(new FileWriter(file));

			final Item[] items = _viewer.getTree().getItems();

			writer.write(String
					.format("Test Description,Result,Progress State,Processing Time [s],Source of Failure,Refactoringtool Description,Failure Message%s",
							SWT.LF));

			for (Item item : items) {
				final ITestElement element = (ITestElement) item.getData();

				writer.write(String.format("%s,%s,%s,%s,%s,%s,%s%s",
						getLabel(element), element.getTestResult(),
						element.getProgressState(true),
						getElapsedTimeInSeconds(element),
						getSourceOfFailure(element),
						getToolDescription(element),
						getFailureMessage(element), SWT.LF));
			}

		} catch (FileNotFoundException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		} catch (IOException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		} finally {
			getResourceUtil().close(writer);
		}
	}

	@Pre("null != element")
	@Post("null != $return")
	private String getToolDescription(final ITestElement element) {
		final Object target = element.getTarget();
		if (null != target && target instanceof RefactoringTestElement) {
			String text;
			final RefactoringTestElement targetTestElement = (RefactoringTestElement) target;
			final RefactoringDescriptorProxy descriptor = targetTestElement
					.getTarget();
			if (null != descriptor) {
				text = descriptor.toString();
			} else {
				text = targetTestElement.getLabel();
			}
			return replaceTokenChars(text);
		}
		return "";
	}

	@Pre("null != element")
	@Post("null != $return")
	private String getElapsedTimeInSeconds(final ITestElement element) {
		return String.format("%.3f", element.getElapsedTimeInSeconds())
				.replace(',', '.');
	}

	@Pre("null != element && null != failures")
	private void collectFailures(final ITestElement element,
			final List<ITestElement> failures) {
		if (element.getTestResult().isSuccess()) {
			return;
		}

		if (element instanceof ITestElementContainer) {
			final ITestElement[] children = ((ITestElementContainer) element)
					.getChildren();
			for (ITestElement child : children) {
				collectFailures(child, failures);
			}
		}

		failures.add(element);
	}

	@Pre("null != element")
	@Post("null != $return")
	private ITestElement[] getFailurePath(final ITestElement element) {
		final List<ITestElement> elements = new ArrayList<ITestElement>();
		collectFailures(element, elements);
		return elements.toArray(new ITestElement[elements.size()]);
	}

	@Pre("null != text")
	@Post("null != $return")
	private String replaceTokenChars(final String text) {
		return text.replace(SWT.LF, '#').replace(',', ';').replace('\t', '|')
				.trim();
	}

	@Pre("null != element")
	@Post("null != $return")
	private String getFailureMessage(final ITestElement element) {
		final ITestElement[] failures = getFailurePath(element);
		for (ITestElement failure : failures) {
			if (failure instanceof JUnitTestCaseElement) {
				return String.format("%s[%s]", replaceTokenChars(failure
						.getLabel()),
						replaceTokenChars(((JUnitTestCaseElement) failure)
								.getStackFrame()));
			}
			if (failure instanceof JavaCompileProblemTestElement) {
				return replaceTokenChars(((JavaCompileProblemTestElement) failure)
						.getLabel());
			}
			if (failure instanceof ExceptionTestElement) {
				return String.format("%s[%s]", replaceTokenChars(failure
						.getLabel()),
						replaceTokenChars(((ExceptionTestElement) failure)
								.getStackFrame()));
			}
		}
		return "";
	}

	@Pre("null != element")
	@Post("null != $return")
	private String getSourceOfFailure(final ITestElement element) {
		final ITestElement[] failures = getFailurePath(element);
		for (ITestElement failure : failures) {
			if (failure instanceof ITestContext) {
				final ITestContext context = (ITestContext) failure;
				final ITestActor actor = context.getState().getOriginator();
				if (actor instanceof IOracle) {
					return replaceTokenChars(actor.getName());
				}
				if (actor instanceof IRefactoringTestAdapter) {
					return replaceTokenChars(actor.getName());
				}
			}
		}
		return "";
	}

	@Pre("null != element")
	@Post("null != $return")
	private String getLabel(final ITestElement element) {
		return replaceTokenChars(TestElementFieldLabel.getText(element));
	}

	private void createActions() {
		_save = createAction(new Action() {
			@Override
			public void run() {
				saveSummary();
			}
		}, "Save Summary", "etool16/save.gif");

		_expandAll = createAction(new Action() {
			@Override
			public void run() {
				_viewer.expandAll();
			}
		}, "Expand All", "etool16/expandall.gif");

		_collapseAll = createAction(new Action() {
			@Override
			public void run() {
				_viewer.collapseAll();
			}
		}, "Collapse All", "etool16/collapseall.gif");

		_autoFullExpand = createAction("Auto Expand", IAction.AS_CHECK_BOX,
				null);

		_rerunAll = createAction(new Action() {
			@Override
			public void run() {
				final TestRootRunSession session = (TestRootRunSession) _viewer.getInput();
				if (null == session
						|| !(session instanceof AbstractTestApplication)) {
					return;
				}

				// Reset test session
				session.setProgressState(ProgressState.NOT_STARTED);

				if (session.getProgressState(true) != ProgressState.NOT_STARTED) {
					getRTTCoreUIPlugin().logWarning(
							"Session could not be reseted");
					return;
				}

				final ProgressMonitorDialog dialog = new ProgressMonitorDialog(
						null);
				final IRunnableWithProgress runnable = new IRunnableWithProgress() {
					public void run(final IProgressMonitor monitor) {
						// Re-run
						try {
							((AbstractTestApplication) session).run(monitor);
						} catch (CoreException e) {
							handleException(e);
						}
					}
				};

				try {
					dialog.run(true, true, runnable);
				} catch (InvocationTargetException e) {
					session.setTestResult(Result.ERROR);
					session.setProgressState(ProgressState.COMPLETED);
					handleException(new CoreException(createError(null, e)));
				} catch (InterruptedException e) {
					session.setProgressState(ProgressState.STOPPED);
					getRTTCoreUIPlugin().logInfo(
							"Operation canceled, through user request");
				}
			}
		}, "Re-run All", "etool16/relaunch.gif");

		_hideSuccess = createAction("Hide Succeeded", IAction.AS_CHECK_BOX,
				new IPropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent event) {
						if (event.getProperty() != IAction.CHECKED) {
							return;
						}

						final ViewerFilter successFilter = TestResultViewFilterHolder._SUCCESS_FILTER;
						final boolean value = (Boolean) event.getNewValue();
						if (value) {
							_viewer.addFilter(successFilter);
						} else {
							_viewer.removeFilter(successFilter);
						}
					}
				});
		_hideIgnored = createAction("Hide Ignored", IAction.AS_CHECK_BOX,
				new IPropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent event) {
						if (event.getProperty() != IAction.CHECKED) {
							return;
						}

						final ViewerFilter filter = TestResultViewFilterHolder._IGNORED_FILTER;
						final boolean value = (Boolean) event.getNewValue();
						if (value) {
							_viewer.addFilter(filter);
						} else {
							_viewer.removeFilter(filter);
						}
					}
				});
		_hideIgnored.setChecked(true);
		_hideErrors = createAction("Hide Errors", IAction.AS_CHECK_BOX,
				new IPropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent event) {
						if (event.getProperty() != IAction.CHECKED) {
							return;
						}

						final ViewerFilter filter = TestResultViewFilterHolder._ERRORS_FILTER;
						final boolean value = (Boolean) event.getNewValue();
						if (value) {
							_viewer.addFilter(filter);
						} else {
							_viewer.removeFilter(filter);
						}
					}
				});
		_hideFailures = createAction("Hide Failures", IAction.AS_CHECK_BOX,
				new IPropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent event) {
						if (event.getProperty() != IAction.CHECKED) {
							return;
						}

						final ViewerFilter filter = TestResultViewFilterHolder._FAILURES_FILTER;
						final boolean value = (Boolean) event.getNewValue();
						if (value) {
							_viewer.addFilter(filter);
						} else {
							_viewer.removeFilter(filter);
						}
					}
				});
		_hideInitialization = createAction("Hide Initialization",
				IAction.AS_CHECK_BOX, new IPropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent event) {
						if (event.getProperty() != IAction.CHECKED) {
							return;
						}

						final ViewerFilter filter = TestResultViewFilterHolder._INITIALIZATION_FILTER;
						final boolean value = (Boolean) event.getNewValue();
						if (value) {
							_viewer.addFilter(filter);
						} else {
							_viewer.removeFilter(filter);
						}
					}
				});
		_hideInitialization.setChecked(true);
	}

	private void fillGeneralActions(final IContributionManager manager) {
		manager.add(_rerunAll);
		manager.add(_save);
		manager.add(new Separator("Display"));
		manager.add(_expandAll);
		manager.add(_collapseAll);
		manager.add(_autoFullExpand);
	}

	private void fillFilterActions(final IContributionManager manager) {
		manager.add(new Separator("Filters"));
		manager.add(_hideSuccess);
		manager.add(_hideErrors);
		manager.add(_hideFailures);
		manager.add(_hideIgnored);
		manager.add(_hideInitialization);
	}

	@Pre("null != actionBars")
	private void createResultToolBar(final IActionBars actionBars) {
		final IToolBarManager toolBarManager = actionBars.getToolBarManager();
		final IMenuManager menuManager = actionBars.getMenuManager();

		fillGeneralActions(menuManager);
		fillFilterActions(menuManager);

		fillGeneralActions(toolBarManager);

		getViewSite().getActionBars().updateActionBars();
	}

	@Pre("null != currentColumns")
	private void createColumns(final TreeViewer viewer,
			final ViewerField[] fields, final TreeColumn[] currentColumns) {
		final Tree tree = viewer.getTree();
		final TableLayout layout = new TableLayout();

		for (int i = 0; i < fields.length; i++) {
			final ViewerField field = fields[i];

			TreeViewerColumn column;
			if (i < currentColumns.length) {
				column = new TreeViewerColumn(viewer, currentColumns[i]);
			} else {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setResizable(field.isResizeable());
				column.getColumn().setMoveable(true);
			}

			column.getColumn().setData(FIELD, field);

			// Show the help in the first column
			column.setLabelProvider(new ViewerFieldLabelProvider(field));
			column.getColumn().setText(field.getColumnHeaderText());
			column.getColumn().setToolTipText(field.getColumnTooltipText());
			column.getColumn().setImage(field.getColumnHeaderImage());

			final EditingSupport support = field.getEditingSupport(viewer);
			if (support != null) {
				column.setEditingSupport(support);
			}

			layout.addColumnData(new ColumnPixelData(field
					.getDefaultColumnWidth(tree), true, true));
		}

		// Remove extra columns
		if (currentColumns.length > fields.length) {
			for (int i = fields.length; i < currentColumns.length; i++) {
				currentColumns[i].dispose();
			}
		}

		viewer.getTree().setLayout(layout);
		tree.setLinesVisible(true);
		tree.setHeaderVisible(true);
		tree.layout(true);
	}

	private static void handleException(final CoreException e) {
		// Check for a nested CoreException
		IStatus status = e.getStatus();
		if (status != null && status.getException() instanceof CoreException) {
			status = ((CoreException) status.getException()).getStatus();
		}

		if (status == null) {
			StatusManager.getManager().handle(createError(e.getMessage(), e),
					StatusManager.SHOW);
		} else {
			StatusManager.getManager().handle(status, StatusManager.SHOW);
		}
	}

	private ITreeContentProvider getContentProvider() {
		try {
			return getDefault().getTestResultContentProvider();
		} catch (CoreException e) {
			handleException(e);
		}
		return null;
	}

	/**
	 * Shows test result view in active workbench page.
	 * 
	 * @return Test result view
	 */
	public static TestResultView showViewInActivePage() {
		return showViewInActivePage(findViewInActivePage());
	}

	/**
	 * Retrieves the view info field.
	 * 
	 * @return Info field
	 */
	public Label getInfoField() {
		return _label;
	}

	private static TestResultView findViewInActivePage() {
		IWorkbenchPage page = RTTCoreUIPlugin.getDefault().getActivePage();
		if (page == null) {
			return null;
		}
		return (TestResultView) page.findView(TestResultView.ID);
	}

	private static TestResultView showViewInActivePage(
			TestResultView testResultView) {
		IWorkbenchPart activePart = null;
		IWorkbenchPage page = null;

		try {
			if (testResultView != null && !testResultView.isDisposed())
				return testResultView;

			page = RTTCoreUIPlugin.getDefault().getActivePage();
			if (page == null) {
				return null;
			}

			activePart = page.getActivePart();
			// show the result view if it isn't shown yet
			return (TestResultView) page.showView(TestResultView.ID);
		} catch (PartInitException e) {
			handleException(e);
			return null;
		} finally {
			// restore focus stolen by the creation of the result view
			if (page != null && activePart != null) {
				page.activate(activePart);
			}
		}
	}

	private void openSelected(final ISelection selection) {
		if (selection != null && selection instanceof IStructuredSelection) {
			final IAction action = getFirstApplicableAction(((IStructuredSelection) selection)
					.toArray());
			if (null != action) {
				action.run();
			}
		}
	}

	private void createResultTreeViewer(final Composite parent, final int hspan) {
		final Tree tree = new Tree(parent, SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.MULTI | SWT.FULL_SELECTION | SWT.LEFT_TO_RIGHT
				| SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = hspan;
		tree.setLayoutData(gd);

		_viewer = new TreeViewer(tree);
		_viewer.getTree().setLinesVisible(true);
		_viewer.setUseHashlookup(true);
		createColumns(_viewer, getResultVisibleFields(), new TreeColumn[0]);

		_viewer.setContentProvider(getContentProvider());
		_viewer.setInput(null);

		getSite().setSelectionProvider(_viewer);

		new OpenAndLinkWithEditorHelper(_viewer) {
			@Override
			protected void activate(ISelection selection) {
				final int currentMode = OpenStrategy.getOpenMethod();
				try {
					OpenStrategy.setOpenMethod(OpenStrategy.DOUBLE_CLICK);
					openSelected(selection);
				} finally {
					OpenStrategy.setOpenMethod(currentMode);
				}
			}

			@Override
			protected void linkToEditor(ISelection selection) {
				// Not supported by this part
			}

			@Override
			protected void open(ISelection selection, boolean activate) {
				openSelected(selection);
			}
		};

		_viewer.getTree().addTreeListener(new TreeAdapter() {
			@Override
			public void treeCollapsed(final TreeEvent e) {
				expandChildren((ITestElementContainer) e.item.getData(), false);
				_viewer.refresh();
			}

			private void expandChildren(ITestElementContainer container,
					boolean fState) {

				final List<ITestElement> toExpand = new ArrayList<ITestElement>();
				final List<ITestElementContainer> stack = new ArrayList<ITestElementContainer>();

				stack.add(container);

				while (!stack.isEmpty()) {
					container = stack.remove(0);
					toExpand.add(container);

					for (ITestElement child : container.getChildren()) {
						if (child instanceof ITestElementContainer
								&& (_autoFullExpand.isChecked() || !child
										.getTestResult().isSuccess())) {
							toExpand.add(child);
							stack.add((ITestElementContainer) child);
						}
					}

				}

				for (ITestElement element : toExpand) {
					_viewer.setExpandedState(element, fState);
				}
			}

			@Override
			public void treeExpanded(final TreeEvent e) {
				expandChildren((ITestElementContainer) e.item.getData(), true);
				_viewer.refresh();
			}
		});

		// Set help on the view itself
		_viewer.getControl().addHelpListener(new HelpListener() {
			@Override
			public void helpRequested(HelpEvent e) {
				Object provider = getAdapter(IContextProvider.class);
				if (provider == null)
					return;

				IContext context = ((IContextProvider) provider)
						.getContext(_viewer.getControl());

				PlatformUI.getWorkbench().getHelpSystem().displayHelp(context);
			}

		});

		_viewer.getTree().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ISelection selection = _viewer.getSelection();
				if (selection instanceof IStructuredSelection) {
					// TODO
				}
			}
		});

		createResultContextMenu();
	}

	@Pre("null != parent")
	@Post("null != $return")
	private Composite createComposite(final Composite parent,
			final int columns, final int hspan, int fill) {
		Composite g = new Composite(parent, SWT.NONE);
		g.setLayout(new GridLayout(columns, false));
		g.setFont(parent.getFont());
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		g.setLayoutData(gd);
		return g;
	}

	@Pre("null != parent && null != font")
	@Post("null != $return")
	private Composite createComposite(final Composite parent, final Font font,
			int style, final int columns, int hspan, int fill, int marginwidth,
			int marginheight) {
		Composite g = new Composite(parent, style);
		GridLayout layout = new GridLayout(columns, false);
		layout.marginWidth = marginwidth;
		layout.marginHeight = marginheight;
		g.setLayout(layout);
		g.setFont(font);
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		g.setLayoutData(gd);
		return g;
	}

	@Pre("null != parent && null != text")
	@Post("null != $return")
	private Label createLabel(final Composite parent, final String text,
			final int hspan, final int fill) {
		Label l = new Label(parent, SWT.BORDER);
		l.setFont(parent.getFont());
		l.setText(text);
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		l.setLayoutData(gd);
		return l;
	}

	@Override
	public void createPartControl(final Composite parent) {
		_fDisposed = false;

		parent.setLayout(new FillLayout());

		final Composite comp = createComposite(parent, 1, 1,
				GridData.FILL_HORIZONTAL);

		final Composite topcomp = createComposite(comp, comp.getFont(),
				SWT.NONE, 1, 1, GridData.FILL_HORIZONTAL, 0, 0);
		_label = createLabel(
				topcomp,
				"Test-Runs: 0  Failures: 0  Errors: 0  Ignored: 0  Time: 0.0s  Progress-State: UNDEFINED",
				1, GridData.FILL_HORIZONTAL);

		final Composite resultcomp = createComposite(comp, comp.getFont(),
				SWT.NONE, 1, 1, GridData.FILL_BOTH, 0, 0);
		createResultTreeViewer(resultcomp, 1);

		createActions();
		createResultToolBar(getViewSite().getActionBars());
	}

	private void createResultContextMenu() {
		// creating the Menu and registering it
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);

		final Menu menu = menuMgr.createContextMenu(_viewer.getControl());
		_viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _viewer);

		menuMgr.setRemoveAllWhenShown(true);

		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				final ISelection selection = getSite().getSelectionProvider()
						.getSelection();
				IAction[] applicables;

				if (selection != null
						&& selection instanceof IStructuredSelection) {
					applicables = getApplicableActions(((IStructuredSelection) selection)
							.toArray());
				} else {
					applicables = new IAction[0];
				}

				for (IAction applicable : applicables) {
					manager.add(applicable);
				}

				// required, for extensions
				menuMgr.add(new Separator(
						IWorkbenchActionConstants.MB_ADDITIONS));
			}
		});
	}

	private IAction getFirstApplicableAction(final Object[] selected) {
		final IAction[] actions = getApplicableActions(selected);
		return (actions.length == 0) ? null : actions[0];
	}

	private IAction[] getApplicableActions(final Object[] selected) {
		try {
			final IActionAdapter[] adapters = RTTCoreUI
					.getActionAdapterProvider().get();
			final List<IAction> actions = new ArrayList<IAction>(
					adapters.length);

			for (IActionAdapter adapter : adapters) {
				if (adapter.isApplicable(selected)) {
					actions.add(adapter.getTarget(selected));
				}
			}

			return actions.toArray(new IAction[actions.size()]);

		} catch (CoreException e) {
			handleException(e);
		}

		return null;
	}

	@Override
	public void setFocus() {
		_viewer.getControl().setFocus();

	}

	/**
	 * Tests if view is disposed.
	 * 
	 * @return True if satisfied otherwise false
	 */
	public boolean isDisposed() {
		return _fDisposed;
	}

	@Override
	public void dispose() {
		_fDisposed = true;
		super.dispose();
	}
}
