package de.fernunihagen.rtt;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;

import org.eclipse.core.resources.IProject;

import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestDescription;

/**
 * Test client session.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class TestClientSession {
	/**
	 * Retrieves the target element filter.
	 * 
	 * @return The target element filter
	 */
	@Post("null != $return")
	public abstract IElementFilter<IAdaptable> getTargetFilter();

	/**
	 * Retrieves the projects in run scope.
	 * 
	 * @return Projects in run scope
	 */
	@Post("null != $return")
	public abstract IProject[] getScope();

	/**
	 * Retrieves the overall test configuration.
	 * 
	 * @return Test configuration
	 */
	@Post("null != $return")
	public abstract ITestConfiguration getConfiguration();

	/**
	 * Retrieves the current actor test configuration.
	 * 
	 * @return Test configuration
	 */
	public abstract ITestConfiguration getActorConfiguration();

	/**
	 * Retrieves the actor name.
	 * 
	 * @return Name
	 */
	@Post("null != $return")
	public abstract String getActorName();

	/**
	 * Retrieves the test actor unique identifier.
	 * 
	 * @return Test actor identifier
	 */
	@Post("null != $return")
	public abstract String getActorID();

	/**
	 * Retrieves the actor test description.
	 * 
	 * @return Test description
	 */
	@Post("null != $return")
	public abstract ITestDescription getDescription();
}
