package de.fernunihagen.rtt.framework;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Abstract test description.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class AbstractTestDescription implements ITestDescription {

	@Invar("null != $target")
	private final Set<String> _list;
	private final boolean _fShouldContinueOnError;

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTCorePlugin().getLogger(
				AbstractTestDescription.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}

	/**
	 * Creates an abstract test run description.
	 * 
	 * @param configuration
	 *            Test configuration
	 */
	@Pre("null != configuration")
	public AbstractTestDescription(final ITestConfiguration configuration) {
		_list = new HashSet<String>();
		_fShouldContinueOnError = configuration.get(
				KEY_ACTORS_ACTOR_CONTINUEONERROR, false);
		final ITestConfiguration[] excluded = configuration
				.getChildren(TYPE_ACTORS_ACTOR_EXCLUDE);

		if (getLogger().isDebugEnabled()) {
			final String message = String.format(
					"Collection excluded resources '%s' (%s)", configuration
							.getID(), excluded.length);
			getLogger().debug(message);
		}

		for (ITestConfiguration exclude : excluded) {
			final String id = exclude.get(KEY_NAME, (String) null);
			if (null != id) {
				getLogger().debug(id);
				_list.add(id);
			}
		}

		getLogger().debug("Collection done");
	}

	/**
	 * Retrieves a list of excluded element identifiers.
	 * 
	 * @return Excluded element identifiers
	 */
	@Post("null != $return")
	public String[] getExcludedElementIdentifiers() {
		return _list.toArray(new String[_list.size()]);
	}

	/**
	 * Retrieves the flag indicating that actor should continue on error.
	 * 
	 * @return true if actor should continue on error
	 */
	public boolean shouldContinueOnError() {
		return _fShouldContinueOnError;
	}
}
