package de.fernunihagen.rtt.framework;

import java.util.ArrayList;
import java.util.List;
import java.util.Collections;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;

import org.eclipse.core.resources.IProject;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Element collector interface, used to select applicable targets.
 * 
 * @param <T>
 *            Type of elements
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IElementCollector<T extends IAdaptable> {

	/**
	 * Null element collector.
	 */
	IElementCollector<IAdaptable> NULL = new IElementCollector<IAdaptable>() {
		@Invar("null != $target")
		private final List<IAdaptable> _matches = Collections
				.unmodifiableList(new ArrayList<IAdaptable>());
		@Invar("null != $target")
		private final IProject[] _emptyScope = new IProject[0];

		@Override
		public void clear() {
		}

		@Override
		public void collect(IProgressMonitor monitor) throws CoreException {
		}

		@Override
		public List<? extends IAdaptable> getMatches() {
			return _matches;
		}

		@Override
		public IProject[] getScope() {
			return _emptyScope;
		}
	};

	/**
	 * Clear collection.
	 */
	void clear();

	/**
	 * Collects matching elements.
	 * 
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	void collect(IProgressMonitor monitor) throws CoreException;

	/**
	 * Retrieves the immutable list of matching elements
	 * 
	 * @return List of matching elements
	 */
	@Post("null != $return")
	List<? extends T> getMatches();

	/**
	 * Retrieves the projects in search scope.
	 * 
	 * @return Projects in search scope
	 */
	@Post("null != $return")
	IProject[] getScope();
}
