package de.fernunihagen.rtt.framework;

import org.eclipse.core.runtime.IAdaptable;

/**
 * Element filter interface.
 * 
 * @author Osama El Hosami
 * 
 * @param <T>
 *            Model element type
 */
public interface IElementFilter<T> {
	/**
	 * Filter accepts no elements.
	 */
	IElementFilter<IAdaptable> NONE = new IElementFilter<IAdaptable>() {
		@Override
		public boolean accept(IAdaptable element) {
			return false;
		}
	};
	/**
	 * Filter accepts all elements.
	 */
	IElementFilter<IAdaptable> ALL = new IElementFilter<IAdaptable>() {
		@Override
		public boolean accept(IAdaptable element) {
			return true;
		}
	};

	/**
	 * Tests weather an element is accepted or not.
	 * 
	 * @return True if satisfied otherwise false
	 */
	boolean accept(T element);
}
