package de.fernunihagen.rtt.framework;

import java.util.Map;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;

/**
 * Test actor provider interface.
 * 
 * @param <T>
 *            Test actor type
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestActorProvider<T extends ITestActor> {
	/**
	 * Retrieves the actor provider identifier.
	 * 
	 * @return Identifier
	 */
	@Post("null != $return")
	String getID();

	/**
	 * Retrieves the immutable map of actors.
	 * 
	 * @return Map of actors
	 */
	@Post("null != $return")
	Map<String, T> getMap();
	
	/**
	 * Retrieves all available actors.
	 * 
	 * @return Actors
	 */
	@Post("null != $return")
	T[] get();

	/**
	 * Retrieves the actor with matching identifier.
	 * 
	 * @param id
	 *            Identifier
	 * 
	 * @return The actor
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If not found</li>
	 *             </ul>
	 */
	@Pre("null != id")
	@Post("null != $return")
	T get(String id) throws CoreException;
}
