package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Test actor state interface, representing a actor run state object.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestActorState {
	/**
	 * Retrieves the actor associated with context object.
	 * 
	 * @return Originator
	 */
	@Post("null != $return")
	ITestActor getOriginator();

	/**
	 * Tests if actor is still active.
	 * 
	 * @return True if active otherwise false
	 */
	boolean isActive();

	/**
	 * Resets state object to state after initialization.
	 */
	void reset();

	/**
	 * Forces initialization of actor state.
	 * 
	 * @param parent
	 *            parent container
	 * @param pm
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != parent")
	void initialize(ITestElementContainer parent, IProgressMonitor pm)
			throws CoreException;
}
