package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IAdaptable;

/**
 * Test run session holding all informations of a test run.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestRunSession extends ITestElementContainer {
	/**
	 * Retrieves the direct target element filter.
	 * 
	 * @return The direct target element filter
	 */
	@Post("null != $return")
	IElementFilter<IAdaptable> getDirectTargetFilter();

	/**
	 * Adds an indirect target element filter.
	 * 
	 * @param filter
	 *            Filter
	 */
	@Pre("null != filter")
	void addIndirectTargetFilter(IElementFilter<IAdaptable> filter);

	/**
	 * Removes an indirect target element filter.
	 * 
	 * @param filter
	 *            Filter
	 */
	@Pre("null != filter")
	void removeIndirectTargetFilter(IElementFilter<IAdaptable> filter);

	/**
	 * Tests if run session is disposed.
	 * 
	 * @return True if disposed otherwise false
	 */
	boolean isDisposed();

	/**
	 * Retrieves the projects in run scope.
	 * 
	 * @return Projects in run scope
	 */
	@Post("null != $return")
	IProject[] getScope();

	/**
	 * Retrieves the overall test configuration.
	 * 
	 * @return Test configuration
	 */
	@Post("null != $return")
	ITestConfiguration getConfiguration();

	/**
	 * Retrieves the test configuration of actor.
	 * 
	 * @param actor
	 *            Test actor
	 * 
	 * @return Test configuration
	 */
	@Post("null != $return")
	ITestConfiguration getConfiguration(ITestActor actor);

	/**
	 * Notifies that element changed.
	 * 
	 * @param element
	 *            Test element
	 * @param oldState
	 *            Old progress state
	 */
	@Pre("null != element && null != oldState")
	void notifyChange(ITestElement element, ProgressState oldState);
}
