package de.fernunihagen.rtt.framework.model;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.ITestElement;

/**
 * Test run event.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestRunEvent {

	/**
	 * Run event type
	 */
	public enum Type {
		STARTED, FINISHED, RESET, UNDEFINED;
	}

	@Invar("null != $target")
	private final ITestElement _element;
	@Invar("null != $target")
	private final Type _type;

	/**
	 * Creates a test run event
	 * 
	 * @param element
	 *            Test element
	 * @param type
	 *            Event type
	 */
	@Pre("null != element && null != type")
	public TestRunEvent(final ITestElement element, final Type type) {
		_element = element;
		_type = type;
	}

	/**
	 * Retrieves event type
	 * 
	 * @return Event type
	 */
	@Post("null != $return")
	public Type getType() {
		return _type;
	}

	/**
	 * Retrieves the test element.
	 * 
	 * @return Test element
	 */
	@Post("null != $return")
	public ITestElement getElement() {
		return _element;
	}
}
