package de.fernunihagen.rtt.framework.model;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElement;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunSession;

/**
 * Test run session.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestRunSession extends AbstractTestRunSession {
	@Invar("null != $target")
	private final ITestElementContainer _parent;
	@Invar("null != $target")
	private final String _label;

	/**
	 * Creates a test run session.
	 * 
	 * @param parent
	 *            Parent session
	 * @param label
	 *            Label
	 */
	@Pre("null != parent")
	public TestRunSession(final ITestElementContainer parent, final String label) {
		super();
		_parent = parent;
		_label = label;

		parent.add(this);
	}

	@Override
	@Pre
	public void notifyChange(final ITestElement element,
			final ProgressState oldState) {
		getSession().notifyChange(element, oldState);
	}

	@Override
	@Post
	@Pre
	public ITestConfiguration getConfiguration(final ITestActor actor) {
		return getSession().getConfiguration(actor);
	}

	@Override
	@Post
	public ITestConfiguration getConfiguration() {
		return getSession().getConfiguration();
	}

	@Override
	@Post
	public ITestElementContainer getParentContainer() {
		return _parent;
	}

	@Override
	@Post
	public String getLabel() {
		return _label;
	}

	@Override
	@Post
	public IProject[] getScope() {
		if (getParentContainer() instanceof ITestContext) {
			return ((ITestContext) getParentContainer()).getScope();
		}
		if (getParentContainer() instanceof ITestRunSession) {
			return ((ITestRunSession) getParentContainer()).getScope();
		}
		return getSession().getScope();
	}

	@Override
	public boolean isDisposed() {
		return getSession().isDisposed();
	}

	@Override
	@Pre
	public void addIndirectTargetFilter(
			final IElementFilter<IAdaptable> filter) {
		getSession().addIndirectTargetFilter(filter);
	}

	@Override
	@Post
	public IElementFilter<IAdaptable> getDirectTargetFilter() {
		return getSession().getDirectTargetFilter();
	}

	@Override
	@Pre
	public void removeIndirectTargetFilter(IElementFilter<IAdaptable> filter) {
		getSession().removeIndirectTargetFilter(filter);
	}
}
