package de.fernunihagen.rtt.framework.oracle;

import java.util.ArrayList;
import java.util.List;

import org.contract4j5.contract.Contract;

import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.TestActorProvider;

import de.fernunihagen.rtt.framework.oracle.IOracle.Type;

/**
 * Oracle provider, manages instances of oracles, which implement the interface
 * {@link IOracle}.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class OracleProvider extends TestActorProvider<IOracle> implements
		IOracleProvider {

	/**
	 * Creates an oracle provider.
	 */
	public OracleProvider() {
		super();
	}

	@Override
	@Pre("null != type")
	@Post("null != $return")
	public IOracle[] getOracles(final Type type) {
		final IOracle[] oracles = get();
		final List<IOracle> filtered = new ArrayList<IOracle>(oracles.length);

		for (IOracle oracle : oracles) {
			if (oracle.getType().isKindOf(type)) {
				filtered.add(oracle);
			}
		}

		return filtered.toArray(new IOracle[filtered.size()]);
	}

	@Override
	@Post("null != $return")
	protected IOracle[] getEmptyArray() {
		return new IOracle[0];
	}
}
