package de.fernunihagen.rtt.framework.repository.team;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;
import org.eclipse.core.resources.IResource;

import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.core.resources.mapping.ResourceTraversal;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.team.core.diff.provider.TwoWayDiff;
import org.eclipse.team.core.subscribers.Subscriber;

import de.fernunihagen.rtt.framework.repository.IDiff;
import de.fernunihagen.rtt.framework.repository.IDiffVisitor;
import de.fernunihagen.rtt.framework.repository.ISubscriber;
import de.fernunihagen.rtt.framework.repository.ISubscriberChangeListener;

/**
 * Team subscriber adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TeamSubscriber implements ISubscriber {

	@Invar("null != $target")
	private final Subscriber _adaptee;

	/**
	 * Creates a team subscriber.
	 * 
	 * @param subscriber
	 *            Subscriber
	 */
	@Pre("null != subscriber")
	public TeamSubscriber(final Subscriber subscriber) {
		_adaptee = subscriber;
	}

	@Override
	@Pre
	public void accept(final ResourceTraversal[] traversals,
			final IDiffVisitor visitor) throws CoreException {
		_adaptee.accept(traversals, new TeamDiffVisitor(visitor));
	}

	@Override
	@Pre
	public void accept(final IResource[] resources, final int depth,
			final IDiffVisitor visitor) throws CoreException {
		_adaptee.accept(resources, depth, new TeamDiffVisitor(visitor));
	}

	@Override
	@Pre
	public void addListener(final ISubscriberChangeListener listener) {
		_adaptee.addListener(new TeamSubscriberChangeListener(listener));
	}

	@Override
	@Pre
	@Post
	public IDiff getDiff(final IResource resource) throws CoreException {
		org.eclipse.team.core.diff.IDiff diff = _adaptee.getDiff(resource);
		if (null == diff) {
			diff = new TwoWayDiff(resource.getLocation(), IDiff.NO_CHANGE, 0);
		}
		return TeamDiff.local(diff);
	}

	@Override
	@Post
	public String getName() {
		return _adaptee.getName();
	}

	@Override
	@Pre
	public int getState(final ResourceMapping mapping, final int stateMask,
			final IProgressMonitor monitor) throws CoreException {
		return _adaptee.getState(mapping, stateMask, monitor);
	}

	@Override
	public boolean isSupervised(final IResource resource) throws CoreException {
		return _adaptee.isSupervised(resource);
	}

	@Override
	@Pre
	@Post
	public IResource[] members(final IResource resource) throws CoreException {
		return _adaptee.members(resource);
	}

	@Override
	@Pre
	public void refresh(final IResource[] resources, final int depth,
			final IProgressMonitor monitor) throws CoreException {
		_adaptee.refresh(resources, depth, monitor);
	}

	@Override
	@Pre
	public void refresh(final ResourceTraversal[] traversals, final IProgressMonitor monitor)
			throws CoreException {
		_adaptee.refresh(traversals, monitor);
	}

	@Override
	@Pre
	public void removeListener(final ISubscriberChangeListener listener) {
		_adaptee.removeListener(new TeamSubscriberChangeListener(listener));
	}

	@Override
	@Post
	public IResource[] roots() {
		return _adaptee.roots();
	}

	@Override
	public int hashCode() {
		return _adaptee.hashCode();
	}

	@Override
	public String toString() {
		return _adaptee.toString();
	}

	@Override
	public boolean equals(final Object other) {
		if (other == this) {
			return true;
		}
		if (null == other) {
			return false;
		}
		return other.equals(_adaptee);
	}
}
