package de.fernunihagen.rtt.framework.repository.team;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.team.core.subscribers.ISubscriberChangeEvent;
import org.eclipse.team.core.subscribers.Subscriber;

import de.fernunihagen.rtt.framework.repository.ISubscriberChangeListener;

/**
 * Team subscriber change listener adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TeamSubscriberChangeListener implements
		org.eclipse.team.core.subscribers.ISubscriberChangeListener {

	@Invar("null != $target")
	private final ISubscriberChangeListener _adaptee;

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTCorePlugin().getLogger(
				TeamSubscriberChangeListener.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}	
	
	/**
	 * Creates a team change listener
	 * 
	 * @param listener
	 *            Change listener
	 */
	@Pre("null != listener")
	public TeamSubscriberChangeListener(ISubscriberChangeListener listener) {
		_adaptee = listener;
	}

	@Override
	public void subscriberResourceChanged(final ISubscriberChangeEvent[] deltas) {
		for (ISubscriberChangeEvent delta : deltas) {
			final IResource res = delta.getResource();
			final Subscriber subscriber = delta.getSubscriber();

			try {
				_adaptee.resourceChanged(TeamDiff
						.local(subscriber.getDiff(res)));
			} catch (CoreException ex) {
				getLogger().trace("Unexpected", ex);
			}
		}
	}

	@Override
	public int hashCode() {
		return _adaptee.hashCode();
	}

	@Override
	public String toString() {
		return _adaptee.toString();
	}

	@Override
	public boolean equals(final Object other) {
		if (other == this) {
			return true;
		}
		if (null == other) {
			return false;
		}
		return other.equals(_adaptee);
	}
}
