package de.fernunihagen.rtt.framework.testadapter;

import org.contract4j5.contract.Post;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.ltk.core.refactoring.RefactoringDescriptorProxy;

/**
 * A strategy defining basic protocol for performing a refactoring
 * 
 * @author Osama El Hosami
 */
public interface IRefactoringStrategy {

	/**
	 * Null refactoring strategy.
	 */
	IRefactoringStrategy NULL = new IRefactoringStrategy() {
		@Override
		public boolean canUndoChange() {
			return false;
		}

		@Override
		public boolean performChange(IProgressMonitor monitor)
				throws CoreException {
			return false;
		}

		@Override
		public void undoChange(IProgressMonitor monitor) throws CoreException {
		}

		@Override
		public String getName() {
			return IRefactoringStrategy.class.getSimpleName();
		}

		@Override
		public RefactoringDescriptorProxy getDescriptor() {
			return null;
		}
	};

	/**
	 * Tests all conditions of wrapped refactoring and performs requested
	 * change, in case conditions could be satisfied.
	 * 
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @return True if refactoring has been executed otherwise false
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs during change operation</li>
	 *             </ul>
	 */
	boolean performChange(IProgressMonitor monitor) throws CoreException;

	/**
	 * Tests if a performed change can be reverted.
	 * 
	 * @return True if satisfied otherwise false
	 */
	boolean canUndoChange();

	/**
	 * Reverts performed change, in case undo is supported.
	 * 
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs during change operation</li>
	 *             </ul>
	 */
	void undoChange(IProgressMonitor monitor) throws CoreException;

	/**
	 * Retrieves the refactoring descriptor.
	 * 
	 * @return Descriptor or null if not executed
	 */
	RefactoringDescriptorProxy getDescriptor();

	/**
	 * Retrieves the refactoring name.
	 * 
	 * @return Name
	 */
	@Post("null != $return")
	String getName();
}