package de.fernunihagen.rtt.framework.testadapter;

import org.contract4j5.contract.Contract;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestContext;

/**
 * Refactoring test adapter interface, provides the basic protocol for creation
 * of refactorings (strategies) for testing purpose.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IRefactoringTestAdapter extends ITestActor {

	/**
	 * Tests if a performed change can be reverted.
	 * 
	 * @param context
	 *            Run context
	 * 
	 * @return True if satisfied otherwise false
	 */
	boolean canUndoChange(ITestContext context);

	/**
	 * Reverts performed change, in case undo is supported.
	 * 
	 * @param context
	 *            Run context
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs during change operation</li>
	 *             </ul>
	 */
	void undoChange(ITestContext context, IProgressMonitor monitor)
			throws CoreException;
}
