package de.fernunihagen.rtt.framework.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

/**
 * Observer interface, used for generic implementation of the Subject-Observer
 * pattern - described in the book "Java Generics and Collections" by
 * <code>Maurice Naftalin</code> and <code>Philip Wadler</code>. <br/>
 * <br/>
 * <code>Note:</code><br/>
 * Even though this interface is defined as generic, it can only be implemented
 * once per class, independent of given template argument combinations. The
 * reason is caused by how java implements generics, through type erasure. In
 * case it is necessary to implements different observers in one class use
 * delegates.
 * 
 * @param <S>
 *            Subject
 * @param <O>
 *            Observer
 * @param <A>
 *            Argument
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IObserver<S extends IObservable<S, O, A>, O extends IObserver<S, O, A>, A> {

	/**
	 * This method is called whenever the observed object is changed. An
	 * application calls an Observable object's notifyObservers method to have
	 * all the object's observers notified of the change.
	 * 
	 * @param o
	 *            The observable object.
	 * @param arg
	 *            An argument passed to the notifyObservers method.
	 */
	@Pre("null != o")
	void update(S o, A arg);
}
