package de.fernunihagen.rtt.framework.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

/**
 * Property change event parameter object, used for listener notification
 * 
 * @author Osama El Hosami
 */
@Contract
public final class PropertyChangeEvent {
	/**
	 * Property key
	 */
	@Invar("null != key")
	public final String key;
	/**
	 * Property value
	 */
	public final Object value;
	/**
	 * Old property value
	 */
	public final Object oldValue;

	/**
	 * Creates a property change event (parameter object)
	 * 
	 * @param key
	 *            Property key
	 * @param value
	 *            Property value
	 * @param oldValue
	 *            Old property value
	 */
	@Pre("null != key")
	public PropertyChangeEvent(final String key,
			final Object value, final Object oldValue) {
		this.key = key;
		this.value = value;
		this.oldValue = oldValue;
	}
}
