package de.fernunihagen.rtt.internal;

import java.io.IOException;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.Plugin;

import com.tools.logging.PluginLogManager;

/**
 * Proxy log manager.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class LogManagerProxy extends AbstractLogManager {
	@Invar("null != $target")
	private final AbstractLogManager _manager;
	@Invar("null != $target")
	private volatile boolean _fOpen;

	/**
	 * Creates a proxy log manager.
	 * 
	 * @param plugin
	 *            Plug-in
	 * @param manager
	 *            Another log manager
	 */
	@Pre("null != manager")
	public LogManagerProxy(final Plugin plugin, AbstractLogManager manager) {
		super(plugin);
		if (manager instanceof LogManagerProxy) {
			manager = ((LogManagerProxy) manager)._manager;
		}
		_manager = manager;
		_fOpen = false;
	}

	@Override
	@Post("$this.isClosed()")
	public void close() throws IOException {
		if (!isClosed()) {
			synchronized (this) {
				if (!isClosed()) {
					_fOpen = !_manager.unhookPlugin(getOwner());
				}
			}
		}
	}

	@Override
	@Post
	protected PluginLogManager getAdaptee() {
		init();
		return _manager.getAdaptee();
	}

	/**
	 * Initializes the log manager.
	 * 
	 * @return True if successful otherwise fail
	 */
	@Override
	@Post("$return == !$this.isClosed()")
	public boolean init() {
		// double checked locking - should work in JVM's greater or equal to
		// version 1.5
		boolean fInit = !isClosed();
		if (!fInit) {
			synchronized (this) {
				fInit = !isClosed();
				if (!fInit) {
					_fOpen = fInit = _manager.hookPlugin(getOwner());
				}
			}
		}
		return fInit;
	}

	@Override
	public boolean isClosed() {
		return !_fOpen;
	}
}
