package de.fernunihagen.rtt.internal.filter;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.IElementFilter;

/**
 * Creates a filter combined with "or" from two other filters.
 * 
 * @author Osama El Hosami
 * 
 * @param <T>
 *            Model element type
 */
@Contract
public final class OrElementFilter<T> implements IElementFilter<T> {
	@Invar("null != $target")
	private final IElementFilter<T> _one;
	@Invar("null != $target")
	private final IElementFilter<T> _other;

	/**
	 * Creates an or combined element filter.
	 * 
	 * @param one
	 *            One filter
	 * @param other
	 *            Another filter
	 */
	@Pre("null != one && null != other")
	public OrElementFilter(final IElementFilter<T> one,
			final IElementFilter<T> other) {
		_one = one;
		_other = other;
	}

	@Override
	public boolean accept(final T element) {
		return _one.accept(element) || _other.accept(element);
	}
}
