package de.fernunihagen.rtt.internal.jdt;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.JavaCore;

import de.fernunihagen.rtt.framework.IElementFilter;

import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;

/**
 * Compilation unit filter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class CompilationUnitFilter implements IElementFilter<IAdaptable> {

	private static final class CompilationUnitFilterHolder {
		public static final IElementFilter<IAdaptable> _DEFAULT = new CompilationUnitFilter();
	}

	private CompilationUnitFilter() {

	}

	/**
	 * Retrieves the shared instance.
	 * 
	 * @return The shared instance
	 */
	public static IElementFilter<IAdaptable> getDefault() {
		return CompilationUnitFilterHolder._DEFAULT;
	}

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	private boolean isJavaFileResource(final IResource resource) {
		return null != resource
				&& JavaCore.isJavaLikeFileName(resource.getName());
	}

	@Pre("null != resource")
	private boolean hasValidResourceAttributes(final IResource resource) {
		return true;
	}

	@Override
	public boolean accept(final IAdaptable element) {
		final IResource resource = as(element, IResource.class);

		if (!isJavaFileResource(resource)
				|| !hasValidResourceAttributes(resource)) {
			return false;
		}

		final IJavaElement javaElement = JavaCore.create(resource);
		return javaElement.getElementType() == IJavaElement.COMPILATION_UNIT;
	}
}
