package de.fernunihagen.rtt.internal.jdt;

import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;

import org.eclipse.jdt.core.IJavaProject;

import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;

/**
 * JUnit 4 test run, used to launch JUnit test cases in a java project using the
 * JUnit 4 loader.
 * 
 * @param <S>
 *            Test run subject type
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JUnit4TestRun<S> extends JUnitTestRun<S> {

	private static final String DEFAULT_VM_ARGUMENTS = "-Xms40m -Xmx256m -XX:MaxPermSize=256m";

	@Invar("null != $target")
	private final IJavaProject _project;
	
	private static ILaunchConfigurationWorkingCopy createLaunchConfiguration(
			final String testName, final IJavaProject project,
			final List<String> testcaseHandles) throws CoreException {
		final ILaunchConfigurationType configType = getLaunchManager()
				.getLaunchConfigurationType(JUnitLaunchConfiguration.ID);

		final ILaunchConfigurationWorkingCopy wc = configType.newInstance(null,
				getLaunchManager().generateUniqueLaunchConfigurationNameFrom(
						testName));

		// FIXME: make VM arguments configurable
		wc.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS,
				DEFAULT_VM_ARGUMENTS);

		wc.setAttribute(IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME,
				"");
		wc.setAttribute(IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME,
				project.getElementName());
		wc.setAttribute(JUnitLaunchConfiguration.ATTR_TESTS, testcaseHandles);
		wc.setAttribute(JUnitLaunchConfiguration.ATTR_KEEPRUNNING, false);
		wc.setAttribute(JUnitLaunchConfiguration.ATTR_TEST_CONTAINER, project
				.getHandleIdentifier());
		wc.setAttribute(JUnitLaunchConfiguration.ATTR_TEST_RUNNER_KIND,
				JUnitLaunchConfiguration.ID_JUNIT4_TEST_KIND);

		return wc;
	}

	/**
	 * Creates a JUnit 4 test run
	 * 
	 * @param testName
	 *            Test name
	 * @param project
	 *            Project
	 * @param testcaseHandles
	 *            List test case handles
	 * @param subject
	 *            Test run subject
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != testName && null != project && null != testcaseHandles")
	public JUnit4TestRun(final String testName, final IJavaProject project,
			final List<String> testcaseHandles, final S subject)
			throws CoreException {
		super(createLaunchConfiguration(testName, project, testcaseHandles),
				subject);
		_project = project;
	}

	@Override
	public IJavaProject getProject() {
		return _project;
	}
}
