package de.fernunihagen.rtt.internal.jdt;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IType;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.model.TestElement;

/**
 * Java test element.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaTestElement extends TestElement {
	@Invar("null != $target")
	private final IJavaElement _element;
	@Invar("null != $target")
	private Result _result;

	/**
	 * Creates a java test element.
	 * 
	 * @param parent
	 *            Parent container
	 * @param element
	 *            Java element
	 */
	@Pre("null != parent && null != element")
	public JavaTestElement(final ITestElementContainer parent,
			final IJavaElement element) {
		super(parent);
		_element = element;
		_result = Result.IGNORED;

		parent.add(this);
	}

	@Override
	public IJavaElement getTarget() {
		return _element;
	}
	
	/**
	 * Sets the test result.
	 * 
	 * @param result
	 *            Test result
	 */
	@Pre("null != result")
	public void setTestResult(final Result result) {
		_result = result;
	}

	@Override
	@Pre
	public void setProgressState(final ProgressState state) {
		super.setProgressState(state);
	}

	@Override
	@Post
	public Result getTestResult() {
		return _result;
	}

	@Override
	@Post
	public String getLabel() {
		switch (_element.getElementType()) {
		case IJavaElement.TYPE:
			return ((IType) _element).getFullyQualifiedName();
		case IJavaElement.FIELD:
		case IJavaElement.METHOD:
			return ((IMember) _element).getDeclaringType()
					.getFullyQualifiedName()
					+ "." + ((IMember) _element).getElementName();
		case IJavaElement.JAVA_MODEL:
		case IJavaElement.JAVA_PROJECT:
		case IJavaElement.PACKAGE_FRAGMENT_ROOT:
		case IJavaElement.PACKAGE_FRAGMENT:
		case IJavaElement.COMPILATION_UNIT:
		case IJavaElement.CLASS_FILE:
		case IJavaElement.INITIALIZER:
		case IJavaElement.PACKAGE_DECLARATION:
		case IJavaElement.IMPORT_CONTAINER:
		case IJavaElement.IMPORT_DECLARATION:
		case IJavaElement.LOCAL_VARIABLE:
		case IJavaElement.TYPE_PARAMETER:
		case IJavaElement.ANNOTATION:
		default:
			return _element.getElementName();
		}
	}
}
