package de.fernunihagen.rtt.internal.runtime;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubMonitor;

import de.fernunihagen.rtt.framework.IElementCollector;
import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElement;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunner;
import de.fernunihagen.rtt.framework.TestActor;

import de.fernunihagen.rtt.framework.ITestElement.ProgressState;
import de.fernunihagen.rtt.framework.ITestElement.Result;

/**
 * Abstract test runner.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class TestRunner extends TestActor implements ITestRunner {

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTCorePlugin().getLogger(
				RefactoringTestRunner.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}

	/**
	 * Creates a test runner.
	 */
	public TestRunner() {

	}

	@Override
	@Post
	public String getTypeName() {
		return "";
	}

	@Pre
	@Post
	@Override
	protected TestRunnerRunState<?> getState(final ITestContext context) {
		return (TestRunnerRunState<?>) super.getState(context);
	}

	@Post
	@Pre
	@Override
	public IElementCollector<? extends IAdaptable> getCollector(
			final IElementFilter<IAdaptable> filter, final IProject[] scope) {
		return IElementCollector.NULL;
	}

	@Pre
	@Override
	public void start(final ITestContext context, final IProgressMonitor pm)
			throws CoreException {
		try {
			final TestRunnerRunState<?> state = getState(context);
			// FIXME: make configurable
			final boolean fShouldContinueOnError = false;
			final String[] loadFailures = state.getLoadFailures();
			final ITestActor[] actors = state.getActors();
			final int max = loadFailures.length + actors.length;
			final ITestElementContainer container = context
					.createChildContainer(max, "Activation");

			final SubMonitor monitor = SubMonitor.convert(pm,
					"Loading test actors(s)", max);

			if (getLogger().isDebugEnabled()) {
				final String message = String.format(
						"Loading test actor(s) (%s)", max);
				getLogger().debug(message);
			}

			container.setProgressState(ProgressState.RUNNING);

			for (String failure : loadFailures) {
				if (monitor.isCanceled()) {
					throw new OperationCanceledException();
				}

				monitor.subTask(failure);
				final ITestElement element = container.createChild(failure,
						Result.FAILURE);

				monitor.worked(1);

				if (getLogger().isDebugEnabled()) {
					final String message = String.format("%s (%s;%s;%.3fs)",
							element, element.getTestResult(), element
									.getProgressState(true), element
									.getElapsedTimeInSeconds());
					getLogger().debug(message);
				}
			}

			for (ITestActor actor : actors) {
				if (monitor.isCanceled()) {
					throw new OperationCanceledException();
				}

				final String name = actor.getName();
				monitor.subTask(name);

				final boolean fOk = (loadFailures.length == 0 || fShouldContinueOnError);
				final Result result = fOk ? Result.OK : Result.IGNORED;
				final ITestElement element = container
						.createChild(name, result);

				monitor.worked(1);

				if (getLogger().isDebugEnabled()) {
					final String message = String.format("%s (%s;%s;%.3fs)",
							element, element.getTestResult(), element
									.getProgressState(true), element
									.getElapsedTimeInSeconds());
					getLogger().debug(message);
				}
			}

			container.setProgressState(ProgressState.COMPLETED);

			if (getLogger().isDebugEnabled()) {
				final String message = String.format("Load done (%s;%s;%.3fs)",
						container.getTestResult(), container
								.getProgressState(true), container
								.getElapsedTimeInSeconds());
				getLogger().debug(message);
			}

		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}
}
