package de.fernunihagen.rtt.internal.testadapter;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;
import org.eclipse.core.runtime.CoreException;

/**
 * Test procedure step visitor.
 * 
 * @author O. El Hosami
 */
@Contract
public interface ITestProcedureStepVisitor {

	/**
	 * Visits an initialization test procedure step.
	 * 
	 * @param step
	 *            Initialization test procedure step
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != step")
	void visit(InitTestProcedureStep step) throws CoreException;

	/**
	 * Visits a run test procedure step.
	 * 
	 * @param step
	 *            Run test procedure step
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != step")
	void visit(RunTestProcedureStep step) throws CoreException;

	/**
	 * Visits a skip test procedure step.
	 * 
	 * @param step
	 *            Skip test procedure step
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != step")
	void visit(SkipTestProcedureStep step) throws CoreException;

	/**
	 * Visits a next test procedure step.
	 * 
	 * @param step
	 *            Next test procedure step
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != step")
	void visit(NextTestProcedureStep step) throws CoreException;
}
