package de.fernunihagen.rtt.internal.util;

import java.io.Serializable;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Abstract cache class.
 * 
 * @author Osama El Hosami
 * 
 * @param <K>
 *            Key type
 * @param <V>
 *            Value type
 */
@Contract
public abstract class AbstractCache<K, V> implements ICache<K, V>, Serializable {
	private static final long serialVersionUID = -8429790149243762345L;

	@Invar("null != $target")
	private transient IFactory<V, ICache<K, V>> _provider;

	/**
	 * Retrieves a synchronized cache adapter.
	 * 
	 * @param <K>
	 *            Key type
	 * @param <V>
	 *            Value type
	 * 
	 * @param cache
	 *            Cache
	 * 
	 * @return Synchronized cache
	 */
	public static <K, V> ICache<K, V> getSynchronizedCache(
			final ICache<K, V> cache) {
		return new SynchronizedCache<K, V>(cache);
	}

	/**
	 * Retrieves the cache value provider
	 * 
	 * @return Cache value provider
	 */
	@Post
	@Override
	public IFactory<V, ICache<K, V>> getProvider() {
		return _provider;
	}

	/**
	 * Sets the cache value provider
	 * 
	 * @param provider
	 *            Cache value provider
	 */
	@Pre
	@Override
	public void setProvider(final IFactory<V, ICache<K, V>> provider) {
		_provider = provider;
	}

	/**
	 * Unsets the cache value provider.
	 * 
	 * @param provider
	 *            Cache value provider
	 */
	@Override
	public void unsetProvider(final IFactory<V, ICache<K, V>> provider) {
		if (provider == getProvider()) {
			_provider = new NullFactory<V, ICache<K, V>>();
		}
	}
}
