package de.fernunihagen.rtt.internal.util;

import java.io.Closeable;
import java.util.Observer;

import org.eclipse.core.resources.IWorkspaceRunnable;

import de.fernunihagen.rtt.framework.util.IObserver;

/**
 * The duck-typing factory, used as facade for delegate factory management.
 * 
 * @author Osama El Hosami
 */
public final class DuckTypingFactory {

	private static final class RunnerHolder {
		private static final Delegator<Runnable> _DEFAULT = Delegator
				.getDelegator(Runnable.class);
	}

	private static final class WorkspaceRunnerHolder {
		private static final Delegator<IWorkspaceRunnable> _DEFAULT = Delegator
				.getDelegator(IWorkspaceRunnable.class);
	}

	private static final class ObserverHolder {
		private static final Delegator<Observer> _DEFAULT = Delegator
				.getDelegator(Observer.class);
	}

	private static final class IObserverHolder {
		@SuppressWarnings("unchecked")
		private static final Delegator<IObserver> _DEFAULT = Delegator
				.getDelegator(IObserver.class);
	}

	private static final class CloseableHolder {
		private static final Delegator<Closeable> _DEFAULT = Delegator
				.getDelegator(Closeable.class);
	}

	private DuckTypingFactory() {

	}

	/**
	 * Retrieves the factory, which can be used for {@link Runnable} delegate
	 * creation.
	 * 
	 * @return Runner delegate factory
	 */
	public static Delegator<Runnable> getRunnableDelegator() {
		return RunnerHolder._DEFAULT;
	}

	/**
	 * Retrieves the factory, which can be used for {@link IWorkspaceRunnable}
	 * delegate creation.
	 * 
	 * @return Workspace runner delegate factory
	 */
	public static Delegator<IWorkspaceRunnable> getWorkspaceRunnableDelegator() {
		return WorkspaceRunnerHolder._DEFAULT;
	}

	/**
	 * Retrieves the factory, which can be used for {@link Observer} delegate
	 * creation.
	 * 
	 * @return Observer delegate factory
	 */
	public static Delegator<Observer> getObserverDelegator() {
		return ObserverHolder._DEFAULT;
	}

	/**
	 * Retrieves the factory, which can be used for {@link IObserver} delegate
	 * creation.
	 * 
	 * @return Observer delegate factory
	 */
	@SuppressWarnings("unchecked")
	public static Delegator<IObserver> getGenericObserverDelegator() {
		return IObserverHolder._DEFAULT;
	}

	/**
	 * Retrieves the factory, which can be used for {@link Closeable} delegate
	 * creation.
	 * 
	 * @return Closeable delegate factory
	 */
	public static Delegator<Closeable> getCloseableDelegator() {
		return CloseableHolder._DEFAULT;
	}
}
