package de.fernunihagen.rtt.internal.util;

import java.util.Enumeration;
import java.util.Iterator;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

/**
 * Class for equal treatment of enumeration and iterator.
 * 
 * @param <E>
 *            Element type
 * 
 * @author Osama El Hosami
 */
@Contract
public final class EnumerationIterator<E> implements Iterator<E> {
	@Invar("null != $target")
	private Enumeration<E> _adaptee;

	@Pre("null != enumeration")
	private EnumerationIterator(final Enumeration<E> enumeration) {
		_adaptee = enumeration;
	}

	/**
	 * Creates an enumeration iterator for given enumerator.
	 * 
	 * @param enumeration
	 *            Underlying enumeration
	 * 
	 * @return Enumeration iterator
	 */
	public static <E> EnumerationIterator<E> createIterator(
			final Enumeration<E> enumeration) {
		return new EnumerationIterator<E>(enumeration);
	}

	/**
	 * Creates an iterable enumeration for given enumerator.
	 * 
	 * @param enumeration
	 *            Underlying enumeration
	 * 
	 * @return Iterable enumeration
	 */
	public static <E> Iterable<E> createIterable(
			final Enumeration<E> enumeration) {
		return new Iterable<E>() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see java.lang.Iterable#iterator()
			 */
			@Override
			public Iterator<E> iterator() {
				return createIterator(enumeration);
			}
		};
	}

	@Override
	public boolean hasNext() {
		return _adaptee.hasMoreElements();
	}

	@Override
	public E next() {
		return _adaptee.nextElement();
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
}
