package de.fernunihagen.rtt.internal.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

/**
 * Abstract file node visitor.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class FileNodeVisitor {

	/**
	 * Visits a file system node.
	 * 
	 * @param node
	 *            File system node
	 */
	@Pre("null != node")
	public final void visit(final AbstractFileNode node) {
		node.accept(this);
	}

	/**
	 * Visits a directory node.
	 * 
	 * @param dir
	 *            Directory system node
	 */
	@Pre("null != node")
	public void visit(final DirectoryNode dir) {
		for (AbstractFileNode node : dir) {
			node.accept(this);
		}
	}

	/**
	 * Visits a file node.
	 * 
	 * @param node
	 *            File node
	 */
	@Pre("null != node")
	abstract public void visit(FileNode node);
}
