package de.fernunihagen.rtt.internal.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.util.IObservable;
import de.fernunihagen.rtt.framework.util.IObserver;

/**
 * A generic observable object (subject) adapter.
 * 
 * @param <S>
 *            Subject interface
 * @param <O>
 *            Observer interface
 * @param <A>
 *            Argument interface
 * 
 * @author Osama El Hosami
 */
@Contract
public final class SubjectAdapter<S extends IObservable<S, O, A>, O extends IObserver<S, O, A>, A>
		extends Subject<S, O, A> {

	@Invar("null != _adaptee")
	private final S _adaptee;

	/**
	 * Constructs a subject adapter
	 * 
	 * @param adaptee
	 *            Adaptee
	 */
	@Pre("null != adaptee")
	public SubjectAdapter(final S adaptee) {
		_adaptee = adaptee;
	}

	@Override
	@Post()
	protected S getThis() {
		return _adaptee;
	}

}
